package Fiteiro.Pessoa;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2002
 * Company:
 * @author
 * @version 1.0
 */

public class RepositorioPessoasLista implements RepositorioPessoas {

    private Pessoa pessoaAtual;
    private RepositorioPessoasLista pessoaProxima;


     /**
     * Possibilita o acesso aos elementos guardados pelo repositorio,
     * em outras classes.
     *
     * @return      Um iterador de pessoas
     *
     */
    public IteradorPessoas getIterador() {
        int qtd;
        IteradorPessoas retorno;
        Pessoa[] array;

        //iniciando a contagem dos elementos.
        RepositorioPessoasLista rpl = this;
        for (qtd = 0; rpl.pessoaAtual != null; qtd++) {
            rpl = rpl.pessoaProxima;
        }

        //iniciando preenchimento
        array = new Pessoa[qtd];
        rpl = this;
        for (int i = 0; i < qtd; i++) {
            array[i] = rpl.pessoaAtual;
            rpl = rpl.pessoaProxima;
        }

        retorno = new IteradorPessoasArray(array);

        return retorno;
    }

    /**
     * Insere um pessoa no repositrio
     * @param       Um pessoa que vai ser inserido
     */
    public void inserir (Pessoa pessoa){
        if (this.pessoaAtual == null) {
                this.pessoaAtual = pessoa;
                pessoaProxima = new RepositorioPessoasLista();
        } else {
               pessoaProxima.inserir(pessoa);
        }
    }


     /**
     * Diz se um pessoa existe ou no em um repositrio
     * @param       Pessoa a ser pesquisada
     * @return      True se o pessoa for encontrada, false caso contrrio
     *
     */
    public boolean existe(Pessoa pessoa){
        boolean resultado;

        if (this.pessoaAtual == null) {
            resultado = false;
        } else {
            if (this.pessoaAtual.equals(pessoa)) {
                resultado = true;
            } else {
                resultado = pessoaProxima.existe(pessoa);
            }
        }
        return resultado;
    }

     /**
     * Exclui pessoa do repositriio
     * @param       pessoa a ser excluido
     */
    public void remover(Pessoa pessoa) throws PessoaNaoEncontradaException {

        if (this.pessoaAtual != null) {
            if (this.pessoaAtual.equals(pessoa) ) {
                this.pessoaAtual = pessoaProxima.pessoaAtual;
                this.pessoaProxima = pessoaProxima.pessoaProxima;
            } else {
                pessoaProxima.remover(pessoa);
            }
        }
        else {
            PessoaNaoEncontradaException e = new PessoaNaoEncontradaException("Pessoa No Encontrada");
            throw e;
        }
    }

    /**
     * Retorna uma referncia para um pessoa do repositrio, podendo
     * assim modific-la.
     * @param       Pessoa a ser pesquisado.
     * @return      Pessoa do repositrio
     */
    public Pessoa procurar(Pessoa pessoa) throws PessoaNaoEncontradaException {
        Pessoa retorno = null;

        if (this.pessoaAtual != null) {
            if (this.pessoaAtual.equals(pessoa)) {
                retorno  = this.pessoaAtual;
           } else {
                retorno = this.pessoaProxima.procurar(pessoa);
           }
        }

        if(retorno == null){
            PessoaNaoEncontradaException e = new PessoaNaoEncontradaException("Pessoa Nao encontrada");
            throw e;
        }

        return retorno;
    }

    /**
     * Atualiza um pessoa.
     * @param atualPessoa           o pessoa que sera procurado na lista
     * @param modificadoPessoa      o pessoa que ira sobrepor o pessoa procurado
     */
    public void atualizar(Pessoa atualPessoa, Pessoa modificadoPessoa)
                                        throws PessoaNaoEncontradaException {
        Pessoa temporarioPessoa;

        temporarioPessoa = this.procurar(atualPessoa);

        temporarioPessoa.setNome(modificadoPessoa.getNome());
        temporarioPessoa.setEndereco(modificadoPessoa.getEndereco());
        temporarioPessoa.setComplemento(modificadoPessoa.getComplemento());
        temporarioPessoa.setCpf(modificadoPessoa.getCpf());
    }

    /**
     * Procura no repositrio todas as pessoas com uma certa
     * string no nome.
     * @param       A string a ser pesquisada
     * @return      Todas as pessoas encontradas, na forma de um repositrio
     */
    public RepositorioPessoas procuraPessoas(String nome)
                                        throws PessoasNaoEncontradasException{

        RepositorioPessoasLista temp;
        RepositorioPessoasLista lista = new RepositorioPessoasLista();

        nome = nome.toUpperCase();
        temp = this;

        while (temp.pessoaAtual != null) {

            if (temp.pessoaAtual.getNome().toUpperCase().indexOf(nome) != -1) {
                lista.inserir(temp.pessoaAtual);
                temp = temp.pessoaProxima;
            } else {
                temp = temp.pessoaProxima;
            }
        }
        if(lista.pessoaAtual == null){
            PessoasNaoEncontradasException e = new PessoasNaoEncontradasException("No H Pessoas Cadastradas com esse Nome");
            throw e;
        }

        return lista;
    }

     /**
     * Procura no repositrio todas as pessoas
     * @return      Todas as pessoas encontradas, na forma de um repositrio
     */
   public RepositorioPessoas listarTodasPessoas() throws NinguemEncontradoException {
        RepositorioPessoasLista temp;
        RepositorioPessoasLista lista = new RepositorioPessoasLista();

        temp = this;

        while(temp.pessoaAtual != null){

           if(true){
                lista.inserir(temp.pessoaAtual);
                temp = temp.pessoaProxima;
           } else {
                temp = temp.pessoaProxima;
           }

        }
        if(lista.pessoaAtual == null){
            NinguemEncontradoException e = new NinguemEncontradoException("No foram encontradas Pessoas");
            throw e;
        }

        return lista;
  }
}
